"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.scrapeURL = exports.extractTextFragments = exports.TextFragment = void 0;
const puppeteer = require("puppeteer");
const utils_1 = require("./utils");
/**
 * Text fragment parameters as defined in https://web.dev/text-fragments/#the-full-syntax
 */
class TextFragment {
    /**
     * If textEnd is missing, it will match the entire textStart string, otherwise
     * it will return all the data from textStart to textEnd excluding prefix and suffix.
     * @param textStart Starting words
     * @param textEnd Ending words (Optional)
     * @param prefix Prefix (Optional). Must end with '-'
     * @param suffix Suffix (Optional). Must start with '-'
     */
    constructor(textStart, textEnd, prefix, suffix) {
        /**
         * Extracts the complete text fragment from a given plaintext source
         * @param body Text content (usually document.body.textContent)
         * @returns Entire text fragment string if found, else null
         */
        this.extract = (body) => {
            const prefix = this.prefix.replace(/([$^*()+\\|?./[\]])/g, '\\$1');
            const suffix = this.suffix.replace(/([$^*()+\\|?./[\]])/g, '\\$1');
            const textStart = this.textStart.replace(/([$^*()+\\|?./[\]])/g, '\\$1');
            let textEnd = this.textEnd.replace(/([$^*()+\\|?./[\]])/g, '\\$1');
            if (textEnd)
                textEnd = `.*?${textEnd}`;
            const regex = new RegExp(`${prefix} *(${textStart}${textEnd}) *${suffix}`, 'gis');
            const match = body.matchAll(regex).next();
            if (match.value && match.value.length == 2)
                return match.value[1].trim();
        };
        this.textStart = textStart;
        this.textEnd = textEnd !== null && textEnd !== void 0 ? textEnd : '';
        this.prefix = (prefix !== null && prefix !== void 0 ? prefix : '').slice(0, -1); // Remove trailing -
        this.suffix = (suffix !== null && suffix !== void 0 ? suffix : '').slice(1); // Remove initial -
    }
}
exports.TextFragment = TextFragment;
;
/**
 * Extracts the percent-decoded text fragments from an URL
 * @param url URL to get text fragments from
 * @returns An array of text fragments, empty if not found
 */
const extractTextFragments = (url) => {
    if (typeof (url) === "string")
        url = new URL(url);
    const hashComponents = url.hash.split(':~:', 2);
    if (hashComponents.length !== 2)
        return [];
    const textFragmentString = hashComponents[1];
    // Get all text fragments
    const textFragments = [];
    for (const match of textFragmentString.matchAll(utils_1.TEXT_FRAGMENT_REGEX)) {
        const [prefix, textStart, textEnd, suffix] = match
            .slice(1)
            .map((text) => text !== undefined ? decodeURIComponent(text) : undefined);
        if (textStart === undefined)
            throw new TypeError('Starting text missing in text fragment');
        textFragments.push(new TextFragment(textStart, textEnd, prefix, suffix));
    }
    return textFragments;
};
exports.extractTextFragments = extractTextFragments;
/**
 * Scrape a URL to extract the text fragments inside the URL's hash
 * @param url URL to scrape and extract text fragments from
 * @param pageWaitMs If waitTillRendered is false, wait for specified milliseconds to load site
 * @param waitTillRendered (Experimental) use with dynamically loaded websites which load content using JS
 * @returns Array of text fragment contents
 */
const scrapeURL = (url, pageWaitMs, waitTillRendered = false) => __awaiter(void 0, void 0, void 0, function* () {
    if (typeof (url) === "string")
        url = new URL(url);
    const textFragments = (0, exports.extractTextFragments)(url);
    if (textFragments.length === 0)
        return [];
    // If textEnd is not defined for all of the fragments,
    // simply return textStart as extracted. Avoids having to scrape.
    const processingRequired = textFragments
        .every((fragment) => fragment.textEnd);
    if (!processingRequired)
        return textFragments.map((textFragment) => textFragment.textStart);
    // Initialize browser
    const browser = yield puppeteer.launch();
    const page = yield browser.newPage();
    yield page.goto(url.toString());
    if (pageWaitMs && !waitTillRendered)
        yield page.waitForTimeout(pageWaitMs);
    if (waitTillRendered)
        yield (0, utils_1.waitTillHTMLRendered)(page);
    // Get the entire text content
    const pageText = yield page.evaluate(() => document.body.textContent);
    yield browser.close();
    return textFragments.map((textFragment) => textFragment.extract(pageText));
});
exports.scrapeURL = scrapeURL;
